@@ -9,17 +9,18 @@ from deap import base, tools
 from deap.base import Toolbox
 
 from sampo.api.genetic_api import ChromosomeType, FitnessFunction, Individual
+from sampo.base import SAMPO
 from sampo.scheduler.genetic.converter import (convert_schedule_to_chromosome, convert_chromosome_to_schedule,
                                                ScheduleGenerationScheme)
-from sampo.scheduler.topological.base import RandomizedTopologicalScheduler
 from sampo.scheduler.lft.base import RandomizedLFTScheduler
+from sampo.scheduler.topological.base import RandomizedTopologicalScheduler
 from sampo.scheduler.utils import WorkerContractorPool
 from sampo.schemas.contractor import Contractor
 from sampo.schemas.graph import GraphNode, WorkGraph
 from sampo.schemas.landscape import LandscapeConfiguration
 from sampo.schemas.resources import Worker
 from sampo.schemas.schedule import Schedule
-from sampo.schemas.schedule_spec import ScheduleSpec
+from sampo.schemas.schedule_spec import ScheduleSpec, WorkSpec
 from sampo.schemas.time import Time
 from sampo.schemas.time_estimator import WorkTimeEstimator, DefaultWorkEstimator
 from sampo.utilities.resource_usage import resources_peaks_sum, resources_costs_sum, resources_sum
@@ -29,12 +30,13 @@ class TimeFitness(FitnessFunction):
     """
     Fitness function that relies on finish time.
     """
+
     def evaluate(self, chromosome: ChromosomeType, evaluator: Callable[[ChromosomeType], Schedule]) \
             -> tuple[int | float]:
         schedule = evaluator(chromosome)
         if schedule is None:
-            return (Time.inf().value, )
-        return (schedule.execution_time.value, )
+            return (Time.inf().value,)
+        return (schedule.execution_time.value,)
 
 
 class SumOfResourcesPeaksFitness(FitnessFunction):
@@ -48,8 +50,8 @@ class SumOfResourcesPeaksFitness(FitnessFunction):
     def evaluate(self, chromosome: ChromosomeType, evaluator: Callable[[ChromosomeType], Schedule]) -> tuple[float]:
         schedule = evaluator(chromosome)
         if schedule is None:
-            return (Time.inf().value, )
-        return (resources_peaks_sum(schedule, self._resources_names), )
+            return (Time.inf().value,)
+        return (resources_peaks_sum(schedule, self._resources_names),)
 
 
 class SumOfResourcesFitness(FitnessFunction):
@@ -60,11 +62,11 @@ class SumOfResourcesFitness(FitnessFunction):
     def __init__(self, resources_names: Iterable[str] | None = None):
         self._resources_names = list(resources_names) if resources_names is not None else None
 
-    def evaluate(self, chromosome: ChromosomeType, evaluator: Callable[[ChromosomeType], Schedule]) -> float:
+    def evaluate(self, chromosome: ChromosomeType, evaluator: Callable[[ChromosomeType], Schedule]) -> tuple[float]:
         schedule = evaluator(chromosome)
         if schedule is None:
-            return Time.inf().value
-        return resources_sum(schedule, self._resources_names)
+            return (Time.inf().value,)
+        return (resources_sum(schedule, self._resources_names),)
 
 
 class TimeWithResourcesFitness(FitnessFunction):
@@ -75,11 +77,11 @@ class TimeWithResourcesFitness(FitnessFunction):
     def __init__(self, resources_names: Iterable[str] | None = None):
         self._resources_names = list(resources_names) if resources_names is not None else None
 
-    def evaluate(self, chromosome: ChromosomeType, evaluator: Callable[[ChromosomeType], Schedule]) -> float:
+    def evaluate(self, chromosome: ChromosomeType, evaluator: Callable[[ChromosomeType], Schedule]) -> tuple[float]:
         schedule = evaluator(chromosome)
         if schedule is None:
-            return Time.inf().value
-        return schedule.execution_time.value + resources_peaks_sum(schedule, self._resources_names)
+            return (Time.inf().value,)
+        return (schedule.execution_time.value + resources_peaks_sum(schedule, self._resources_names),)
 
 
 class DeadlineResourcesFitness(FitnessFunction):
@@ -97,9 +99,9 @@ class DeadlineResourcesFitness(FitnessFunction):
             -> tuple[int | float]:
         schedule = evaluator(chromosome)
         if schedule is None:
-            return (Time.inf().value, )
+            return (Time.inf().value,)
         return (resources_peaks_sum(schedule, self._resources_names) \
-                    * max(1.0, schedule.execution_time.value / self._deadline.value), )
+                * max(1.0, schedule.execution_time.value / self._deadline.value),)
 
 
 class DeadlineCostFitness(FitnessFunction):
@@ -117,9 +119,9 @@ class DeadlineCostFitness(FitnessFunction):
             -> tuple[int | float]:
         schedule = evaluator(chromosome)
         if schedule is None:
-            return (Time.inf().value, )
+            return (Time.inf().value,)
         return (resources_costs_sum(schedule, self._resources_names) \
-                * max(1.0, schedule.execution_time.value / self._deadline.value), )
+                * max(1.0, schedule.execution_time.value / self._deadline.value),)
 
 
 class TimeAndResourcesFitness(FitnessFunction):
@@ -159,9 +161,11 @@ def init_toolbox(wg: WorkGraph,
                  contractor2index: dict[str, int],
                  contractor_borders: np.ndarray,
                  node_indices: list[int],
+                 priorities: list[int],
                  parents: dict[int, set[int]],
                  children: dict[int, set[int]],
                  resources_border: np.ndarray,
+                 contractors_available: np.ndarray,
                  assigned_parent_time: Time = Time(0),
                  fitness_weights: tuple[int | float, ...] = (-1,),
                  work_estimator: WorkTimeEstimator = DefaultWorkEstimator(),
@@ -193,21 +197,22 @@ def init_toolbox(wg: WorkGraph,
     selection = tools.selNSGA2 if is_multiobjective else select_new_population
     toolbox.register('select', selection, k=selection_size)
     # combined crossover
-    toolbox.register('mate', mate, rand=rand, toolbox=toolbox)
+    toolbox.register('mate', mate, rand=rand, toolbox=toolbox, priorities=priorities)
     # combined mutation
     toolbox.register('mutate', mutate, order_mutpb=mut_order_pb, res_mutpb=mut_res_pb, zone_mutpb=mut_zone_pb,
                      rand=rand, parents=parents, children=children, resources_border=resources_border,
-                     statuses_available=statuses_available)
+                     contractors_available=contractors_available, statuses_available=statuses_available,
+                     priorities=priorities)
     # crossover for order
-    toolbox.register('mate_order', mate_scheduling_order, rand=rand, toolbox=toolbox)
+    toolbox.register('mate_order', mate_scheduling_order, rand=rand, toolbox=toolbox, priorities=priorities)
     # mutation for order
     toolbox.register('mutate_order', mutate_scheduling_order, mutpb=mut_order_pb, rand=rand, parents=parents,
-                     children=children)
+                     children=children, priorities=priorities)
     # crossover for resources
     toolbox.register('mate_resources', mate_resources, rand=rand, toolbox=toolbox)
     # mutation for resources
     toolbox.register('mutate_resources', mutate_resources, resources_border=resources_border,
-                     mutpb=mut_res_pb, rand=rand)
+                     contractors_available=contractors_available, mutpb=mut_res_pb, rand=rand)
     # mutation for resource borders
     toolbox.register('mutate_resource_borders', mutate_resource_borders, contractor_borders=contractor_borders,
                      mutpb=mut_res_pb, rand=rand)
@@ -216,7 +221,7 @@ def init_toolbox(wg: WorkGraph,
                      statuses_available=landscape.zone_config.statuses.statuses_available())
 
     toolbox.register('validate', is_chromosome_correct, node_indices=node_indices, parents=parents,
-                     contractor_borders=contractor_borders)
+                     contractor_borders=contractor_borders, index2node=index2node, index2contractor=index2contractor_obj)
     toolbox.register('schedule_to_chromosome', convert_schedule_to_chromosome,
                      work_id2index=work_id2index, worker_name2index=worker_name2index,
                      contractor2index=contractor2index, contractor_borders=contractor_borders, spec=spec,
@@ -275,17 +280,20 @@ def generate_chromosomes(n: int,
 
     def randomized_init(is_topological: bool = False) -> ChromosomeType:
         if is_topological:
-            schedule = RandomizedTopologicalScheduler(work_estimator, int(rand.random() * 1000000)) \
-                .schedule(wg, contractors, spec, landscape=landscape)[0]
+            seed = int(rand.random() * 1000000)
+            schedule, _, _, node_order = RandomizedTopologicalScheduler(work_estimator,
+                                                                        seed).schedule_with_cache(wg, contractors, spec,
+                                                                                                  landscape=landscape)[0]
         else:
-            schedule = RandomizedLFTScheduler(work_estimator=work_estimator, rand=rand).schedule(wg, contractors, spec,
-                                                                                                 landscape=landscape)[0]
-        return convert_schedule_to_chromosome(work_id2index, worker_name2index,
-                                              contractor2index, contractor_borders, schedule, spec, landscape)
+            schedule, _, _, node_order = RandomizedLFTScheduler(work_estimator=work_estimator,
+                                                                rand=rand).schedule_with_cache(wg, contractors, spec,
+                                                                                               landscape=landscape)[0]
+        return convert_schedule_to_chromosome(work_id2index, worker_name2index, contractor2index, contractor_borders,
+                                              schedule, spec, landscape, node_order)
 
     if only_lft_initialization:
-        chromosomes = [randomized_init(is_topological=False) for _ in range(n - 1)]
-        chromosomes.append(init_chromosomes['lft'][0])
+        chromosomes = [toolbox.Individual(randomized_init(is_topological=False)) for _ in range(n - 1)]
+        chromosomes.append(toolbox.Individual(init_chromosomes['lft'][0]))
         return chromosomes
 
     count_for_specified_types = (n // 3) // len(init_chromosomes)
@@ -345,11 +353,12 @@ def generate_chromosome(wg: WorkGraph,
     """
 
     def randomized_init() -> ChromosomeType:
-        schedule = RandomizedTopologicalScheduler(work_estimator,
+        schedule, _, _, node_order = RandomizedTopologicalScheduler(work_estimator,
                                                   int(rand.random() * 1000000)) \
-            .schedule(wg, contractors, spec, landscape=landscape)[0]
+            .schedule_with_cache(wg, contractors, spec, landscape=landscape)[0]
         return convert_schedule_to_chromosome(work_id2index, worker_name2index,
-                                              contractor2index, contractor_borders, schedule, spec, landscape)
+                                              contractor2index, contractor_borders,
+                                              schedule, spec, landscape, node_order)
 
     chance = rand.random()
     if chance < 0.2:
@@ -380,15 +389,16 @@ def select_new_population(population: list[Individual], k: int) -> list[Individu
 
 
 def is_chromosome_correct(ind: Individual, node_indices: list[int], parents: dict[int, set[int]],
-                          contractor_borders: np.ndarray) -> bool:
+                          contractor_borders: np.ndarray, index2node: dict[int, GraphNode],
+                          index2contractor: dict[int, Contractor]) -> bool:
     """
     Check correctness of works order and contractors borders.
     """
-    return is_chromosome_order_correct(ind, parents) and \
-        is_chromosome_contractors_correct(ind, node_indices, contractor_borders)
+    return is_chromosome_order_correct(ind, parents, index2node) and \
+        is_chromosome_contractors_correct(ind, node_indices, contractor_borders, index2node, index2contractor)
 
 
-def is_chromosome_order_correct(ind: Individual, parents: dict[int, set[int]]) -> bool:
+def is_chromosome_order_correct(ind: Individual, parents: dict[int, set[int]], index2node: dict[int, GraphNode]) -> bool:
     """
     Checks that assigned order of works are topologically correct.
     """
@@ -399,17 +409,37 @@ def is_chromosome_order_correct(ind: Individual, parents: dict[int, set[int]]) -
         if not parents[work_index].issubset(used):
             # logger.error(f'Order validation failed: {work_order}')
             return False
+
+        # validate priorities
+        work_node = index2node[work_index]
+        if any(index2node[parent].work_unit.priority > work_node.work_unit.priority for parent in parents[work_index]):
+            # SAMPO.logger.error(f'Order validation failed')
+            return False
     return True
 
 
 def is_chromosome_contractors_correct(ind: Individual, work_indices: Iterable[int],
-                                      contractor_borders: np.ndarray) -> bool:
+                                      contractor_borders: np.ndarray,
+                                      index2node: dict[int, GraphNode],
+                                      index2contractor: dict[int, Contractor]) -> bool:
     """
     Checks that assigned contractors can supply assigned workers.
     """
     if not work_indices:
         return True
+
+    order = ind[0]
     resources = ind[1][work_indices]
+
+    # check contractor align with the spec
+    spec: ScheduleSpec = ind[3]
+    contractors = resources[:, -1]
+    for i in range(len(order)):
+        work_index = order[i]
+        work_spec = spec[index2node[work_index].id]
+        if not work_spec.is_contractor_enabled(index2contractor[contractors[work_index]].id):
+            return False
+
     # sort resource part of chromosome by contractor ids
     resources = resources[resources[:, -1].argsort()]
     # get unique contractors and indexes where they start
@@ -434,7 +464,7 @@ def get_order_part(order: np.ndarray, other_order: np.ndarray) -> np.ndarray:
 
 
 def mate_scheduling_order(ind1: Individual, ind2: Individual, rand: random.Random,
-                          toolbox: Toolbox, copy: bool = True) -> tuple[Individual, Individual]:
+                          toolbox: Toolbox, priorities: np.ndarray, copy: bool = True) -> tuple[Individual, Individual]:
     """
     Two-Point crossover for order.
 
@@ -442,26 +472,40 @@ def mate_scheduling_order(ind1: Individual, ind2: Individual, rand: random.Rando
     :param ind2: second individual
     :param rand: the rand object used for randomized operations
     :param toolbox: toolbox
+    :param priorities: node priorities
     :param copy: if True individuals will be copied before mating so as not to change them
 
     :return: two mated individuals
     """
     child1, child2 = (toolbox.copy_individual(ind1), toolbox.copy_individual(ind2)) if copy else (ind1, ind2)
 
+    def mate_parts(part1, part2):
+        parent1 = part1.copy()
+
+        min_mating_amount = len(part1) // 4
+
+        two_point_order_crossover(part1, part2, min_mating_amount, rand)
+        two_point_order_crossover(part2, parent1, min_mating_amount, rand)
+
     order1, order2 = child1[0], child2[0]
-    parent1 = ind1[0].copy()
 
-    min_mating_amount = len(order1) // 4
+    # mate parts inside priority groups
 
-    two_point_order_crossover(order1, order2, min_mating_amount, rand)
-    two_point_order_crossover(order2, parent1, min_mating_amount, rand)
+    # priorities of tasks with same order-index should be the same (if chromosome is valid)
+    cur_priority = priorities[order1[0]]
+    cur_priority_group_start = 0
+    for i in range(len(order1)):
+        if priorities[order1[i]] != cur_priority:
+            cur_priority = priorities[order1[i]]
+
+            mate_parts(order1[cur_priority_group_start:i], order2[cur_priority_group_start:i])
 
     return toolbox.Individual(child1), toolbox.Individual(child2)
 
 
 def two_point_order_crossover(child: np.ndarray, other_parent: np.ndarray, min_mating_amount: int, rand: random.Random):
     """
-    This faction realizes Two-Point crossover for order.
+    This function implements Two-Point crossover for order.
 
     :param child: order to which implements crossover, it is equal to order of first parent.
     :param other_parent: order of second parent from which mating part will be taken.
@@ -486,20 +530,8 @@ def two_point_order_crossover(child: np.ndarray, other_parent: np.ndarray, min_m
     return child
 
 
-def mutate_scheduling_order(ind: Individual, mutpb: float, rand: random.Random,
-                            parents: dict[int, set[int]], children: dict[int, set[int]]) -> Individual:
-    """
-    Mutation operator for works scheduling order.
-
-    :param ind: the individual to be mutated
-    :param mutpb: probability of gene mutation
-    :param rand: the rand object used for randomized operations
-    :param parents: mapping object of works and their parent-works to create valid order
-    :param children: mapping object of works and their children-works to create valid order
-
-    :return: mutated individual
-    """
-    order = ind[0]
+def mutate_scheduling_order_core(order: np.ndarray, mutpb: float, rand: random.Random,
+                                 parents: dict[int, set[int]], children: dict[int, set[int]]):
     # number of possible mutations = number of works except start and finish works
     num_possible_muts = len(order) - 2
     # generate mask of works to mutate based on mutation probability
@@ -518,12 +550,12 @@ def mutate_scheduling_order(ind: Individual, mutpb: float, rand: random.Random,
             # find max index of parent of the current work
             # +1 because insertion should be righter
             i_parent = np.max(np.where(np.isin(order[:i], list(parents[work]), assume_unique=True))[0],
-                              initial=i - 1) + 1
+                              initial=0) + 1
             # find min index of child of the current work
             # +i because the slice [i + 1:] was taken, and +1 is not needed because these indexes will be shifted left
             # after current work deletion
             i_children = np.min(np.where(np.isin(order[i + 1:], list(children[work]), assume_unique=True))[0],
-                                initial=0) + i
+                                initial=len(order) - 2 - i) + i
             if i_parent == i_children:
                 # if child and parent indexes are equal then no mutation can be done
                 continue
@@ -544,6 +576,38 @@ def mutate_scheduling_order(ind: Individual, mutpb: float, rand: random.Random,
                 # after the current work insertion in new generated index
                 indexes_of_works_to_mutate[indexes_of_works_to_mutate >= new_i] += 1
 
+
+def mutate_scheduling_order(ind: Individual, mutpb: float, rand: random.Random, priorities: np.ndarray,
+                            parents: dict[int, set[int]], children: dict[int, set[int]]) -> Individual:
+    """
+    Mutation operator for works scheduling order.
+
+    :param ind: the individual to be mutated
+    :param mutpb: probability of gene mutation
+    :param rand: the rand object used for randomized operations
+    :param priorities: node priorities
+    :param parents: mapping object of works and their parent-works to create valid order
+    :param children: mapping object of works and their children-works to create valid order
+
+    :return: mutated individual
+    """
+    order = ind[0]
+
+    # priority_groups_count = len(set(priorities))
+    mutpb_for_priority_group = mutpb  # / priority_groups_count
+
+    # priorities of tasks with same order-index should be the same (if chromosome is valid)
+    cur_priority = priorities[order[0]]
+    cur_priority_group_start = 0
+    for i in range(len(order)):
+        if priorities[order[i]] != cur_priority:
+            mutate_scheduling_order_core(order[cur_priority_group_start:i],
+                                         mutpb_for_priority_group,
+                                         rand, parents, children)
+
+            cur_priority = priorities[order[i]]
+            cur_priority_group_start = i
+
     return ind
 
 
@@ -584,15 +648,17 @@ def mate_resources(ind1: Individual, ind2: Individual, rand: random.Random,
 
 
 def mutate_resources(ind: Individual, mutpb: float, rand: random.Random,
-                     resources_border: np.ndarray) -> Individual:
+                     resources_border: np.ndarray,
+                     contractors_available: np.ndarray) -> Individual:
     """
     Mutation function for resources.
     It changes selected numbers of workers in random work in a certain interval for this work.
 
     :param ind: the individual to be mutated
-    :param resources_border: low and up borders of resources amounts
     :param mutpb: probability of gene mutation
     :param rand: the rand object used for randomized operations
+    :param resources_border: low and up borders of resources amounts
+    :param contractors_available: mask of contractors available to do tasks
 
     :return: mutated individual
     """
@@ -604,9 +670,22 @@ def mutate_resources(ind: Individual, mutpb: float, rand: random.Random,
         mask = np.array([rand.random() < mutpb for _ in range(num_works)])
         if mask.any():
             # generate new contractors in the number of received True values of mask
-            new_contractors = np.array([rand.randint(0, num_contractors - 1) for _ in range(mask.sum())])
+
+            # [rand.randint(0, num_contractors - 1) for _ in range(mask.sum())]
+            new_contractors_list = []
+
+            # TODO Rewrite to numpy functions if heavy
+            for task, task_selected in enumerate(mask):
+                if not task_selected:
+                    continue
+                contractors_to_select = np.where(contractors_available[task] == 1)
+                new_contractors_list.append(rand.choices(contractors_to_select[0], k=1)[0])
+
+            new_contractors = np.array(new_contractors_list)
+
             # obtain a new mask of correspondence
             # between the borders of the received contractors and the assigned resources
+
             contractor_mask = (res[mask, :-1] <= ind[2][new_contractors]).all(axis=1)
             # update contractors by received mask
             new_contractors = new_contractors[contractor_mask]
@@ -646,7 +725,8 @@ def mutate_resources(ind: Individual, mutpb: float, rand: random.Random,
     return ind
 
 
-def mate(ind1: Individual, ind2: Individual, optimize_resources: bool, rand: random.Random, toolbox: Toolbox) \
+def mate(ind1: Individual, ind2: Individual, optimize_resources: bool,
+         rand: random.Random, toolbox: Toolbox, priorities: np.ndarray) \
         -> tuple[Individual, Individual]:
     """
     Combined crossover function of Two-Point crossover for order, One-Point crossover for resources
@@ -656,11 +736,12 @@ def mate(ind1: Individual, ind2: Individual, optimize_resources: bool, rand: ran
     :param ind2: second individual
     :param optimize_resources: if True resource borders should be changed after mating
     :param rand: the rand object used for randomized operations
+    :param priorities: priorities
     :param toolbox: toolbox
 
     :return: two mated individuals
     """
-    child1, child2 = mate_scheduling_order(ind1, ind2, rand, toolbox, copy=True)
+    child1, child2 = mate_scheduling_order(ind1, ind2, rand, toolbox, priorities, copy=True)
     child1, child2 = mate_resources(child1, child2, rand, optimize_resources, toolbox, copy=False)
     # TODO Make better crossover for zones and uncomment this
     # child1, child2 = mate_for_zones(child1, child2, rand, copy=False)
@@ -668,9 +749,9 @@ def mate(ind1: Individual, ind2: Individual, optimize_resources: bool, rand: ran
     return toolbox.Individual(child1), toolbox.Individual(child2)
 
 
-def mutate(ind: Individual, resources_border: np.ndarray, parents: dict[int, set[int]],
-           children: dict[int, set[int]], statuses_available: int,
-           order_mutpb: float, res_mutpb: float, zone_mutpb: float,
+def mutate(ind: Individual, resources_border: np.ndarray, contractors_available: np.ndarray,
+           parents: dict[int, set[int]], children: dict[int, set[int]], statuses_available: int,
+           priorities: np.ndarray, order_mutpb: float, res_mutpb: float, zone_mutpb: float,
            rand: random.Random) -> Individual:
     """
     Combined mutation function of mutation for order, mutation for resources and mutation for zones.
@@ -687,8 +768,8 @@ def mutate(ind: Individual, resources_border: np.ndarray, parents: dict[int, set
 
     :return: mutated individual
     """
-    mutant = mutate_scheduling_order(ind, order_mutpb, rand, parents, children)
-    mutant = mutate_resources(mutant, res_mutpb, rand, resources_border)
+    mutant = mutate_scheduling_order(ind, order_mutpb, rand, priorities, parents, children)
+    mutant = mutate_resources(mutant, res_mutpb, rand, resources_border, contractors_available)
     # TODO Make better mutation for zones and uncomment this
     # mutant = mutate_for_zones(mutant, statuses_available, zone_mutpb, rand)
 
