@@ -4,8 +4,9 @@ from typing import Optional, Union
 from sortedcontainers import SortedList
 
 from sampo.scheduler.timeline.base import Timeline
-from sampo.scheduler.timeline.from_start_material_timeline import FromStartSupplyTimeline
+from sampo.scheduler.timeline.hybrid_supply_timeline import HybridSupplyTimeline
 from sampo.scheduler.timeline.zone_timeline import ZoneTimeline
+from sampo.scheduler.timeline.utils import get_exec_times_from_assigned_time_for_chain
 from sampo.scheduler.utils import WorkerContractorPool
 from sampo.schemas.contractor import Contractor
 from sampo.schemas.graph import GraphNode
@@ -67,7 +68,7 @@ class MomentumTimeline(Timeline):
 
         # internal index, earlier - task_index parameter for schedule method
         self._task_index = 0
-        self._material_timeline = FromStartSupplyTimeline(landscape)
+        self._material_timeline = HybridSupplyTimeline(landscape)
         self.zone_timeline = ZoneTimeline(landscape.zone_config)
 
     def find_min_start_time_with_additional(self,
@@ -138,14 +139,14 @@ class MomentumTimeline(Timeline):
             cur_start_time = max_parent_time
             found_earliest_time = False
             while not found_earliest_time:
+                cur_start_time = self._find_min_start_time(self._timeline[contractor_id], inseparable_chain, spec,
+                                                           cur_start_time, exec_time, worker_team)
+
                 material_time = self._material_timeline.find_min_material_time(node,
                                                                                cur_start_time,
                                                                                node.work_unit.need_materials())
-
-                cur_start_time = self._find_min_start_time(self._timeline[contractor_id], inseparable_chain, spec,
-                                                           material_time, exec_time, worker_team)
-
-                if cur_start_time > material_time:
+                if material_time > cur_start_time:
+                    cur_start_time = material_time
                     continue
 
                 zone_time = self.zone_timeline.find_min_start_time(node.work_unit.zone_reqs, cur_start_time,
@@ -402,8 +403,7 @@ class MomentumTimeline(Timeline):
             self.find_min_start_time_with_additional(node, workers, node2swork, spec, assigned_start_time,
                                                      assigned_parent_time, work_estimator)
         if assigned_time is not None:
-            exec_times = {n: (Time(0), assigned_time // len(inseparable_chain))
-                          for n in inseparable_chain}
+            exec_times = get_exec_times_from_assigned_time_for_chain(inseparable_chain, assigned_time)
 
         # TODO Decide how to deal with exec_times(maybe we should remove using pre-computed exec_times)
         self._schedule_with_inseparables(node, node2swork, inseparable_chain, spec,
@@ -417,13 +417,17 @@ class MomentumTimeline(Timeline):
                                     worker_team: list[Worker],
                                     contractor: Contractor,
                                     start_time: Time,
-                                    exec_times: dict[GraphNode, tuple[Time, Time]]):
+                                    exec_times: dict[GraphNode, tuple[Time, Time]],
+                                    work_estimator: WorkTimeEstimator = DefaultWorkEstimator()):
         # 6. create a schedule entry for the task
         # nodes_start_times = {ins_node: ins_node.min_start_time(node2swork) for ins_node in inseparable_chain}
 
         curr_time = start_time
         for i, chain_node in enumerate(inseparable_chain):
-            node_lag, node_time = exec_times[chain_node]
+            if chain_node in exec_times:
+                node_lag, node_time = exec_times[chain_node]
+            else:
+                node_lag, node_time = 0, work_estimator.estimate_time(chain_node.work_unit, worker_team)
 
             # lag_req = nodes_start_times[chain_node] - curr_time
             # node_lag = lag_req if lag_req > 0 else 0
@@ -431,10 +435,9 @@ class MomentumTimeline(Timeline):
             start_work = curr_time + node_lag
             deliveries, mat_del_time = self._material_timeline.deliver_resources(chain_node,
                                                                                  start_work,
-                                                                                 chain_node.work_unit.need_materials(),
-                                                                                 True)
+                                                                                 chain_node.work_unit.need_materials())
             start_work = max(start_work, mat_del_time)
-            self._validate(start_work + node_time, node_time, worker_team)
+            # self._validate(start_work + node_time, node_time, worker_team)
             swork = ScheduledWork(
                 work_unit=chain_node.work_unit,
                 start_end_time=(start_work, start_work + node_time),
