@@ -56,8 +56,11 @@ class GraphNode(JSONSerializable['GraphNode']):
                  parent_works: list['GraphNode'] | list[tuple['GraphNode', float, EdgeType]]):
         self._work_unit = work_unit
         self._parent_edges = []
-        self.add_parents(parent_works)
         self._children_edges = []
+        self.add_parents(parent_works)
+
+        if any(w.id == self.id for w in self.children):
+            raise ValueError('Cycle')
 
     def __hash__(self) -> int:
         return hash(self.id)
@@ -312,7 +315,9 @@ def get_finish_stage(parents: list[GraphNode | tuple[GraphNode, float, EdgeType]
     :return: desired node
     """
     work_id = work_id or uuid_str(rand)
-    work = WorkUnit(str(work_id), 'finish of project', [], group='service_works', is_service_unit=True)
+    finish_priority = max(node.work_unit.priority for node in parents)
+    work = WorkUnit(str(work_id), 'finish of project', [], group='service_works',
+                    priority=finish_priority, is_service_unit=True)
     return GraphNode(work, parents)
 
 
@@ -357,9 +362,9 @@ class WorkGraph(JSONSerializable['WorkGraph']):
         # Define the format of the output DataFrame
         graph_df_structure = {'activity_id': [],
                               'activity_name': [],
-                              'granular_name': [],
+                              'model_name': [],
                               'volume': [],
-                              'measurement': [],
+                              'priority': [],
                               'predecessor_ids': [],
                               'connection_types': [],
                               'lags': []}
@@ -379,9 +384,10 @@ class WorkGraph(JSONSerializable['WorkGraph']):
                 # Get information about tasks from graph nodes (work units)
                 graph_df_structure['activity_id'].append(node_info_dict['work_unit']['id'])
                 graph_df_structure['activity_name'].append(node_info_dict['work_unit']['display_name'])
-                graph_df_structure['granular_name'].append(node_info_dict['work_unit']['name'])
+                # model_name must be string!
+                graph_df_structure['model_name'].append(str(node_info_dict['work_unit']['model_name']))
                 graph_df_structure['volume'].append(node_info_dict['work_unit']['volume'])
-                graph_df_structure['measurement'].append(node_info_dict['work_unit']['volume_type'])
+                graph_df_structure['priority'].append(node_info_dict['work_unit']['priority'])
 
                 if save_req:
                     min_req_dict = dict()
