@@ -1,7 +1,8 @@
 from sampo.schemas.contractor import Contractor
-from sampo.schemas.graph import WorkGraph
+from sampo.schemas.graph import WorkGraph, GraphNode
 from sampo.schemas.schedule import Schedule
 from sampo.schemas.serializable import AutoJSONSerializable
+
 from sampo.utilities.serializers import custom_serializer
 
 
@@ -28,10 +29,16 @@ class ScheduledProject(AutoJSONSerializable['ScheduledProject']):
         self.contractors = contractors
 
     @custom_serializer('contractors')
-    def serialize_contractors(self, value):
+    def serialize_contractors(self, value) -> list:
         return [v._serialize() for v in value]
 
     @classmethod
     @custom_serializer('contractors', deserializer=True)
-    def deserialize_equipment(cls, value):
+    def deserialize_equipment(cls, value) -> list[Contractor]:
         return [Contractor._deserialize(v) for v in value]
+
+    def critical_path(self) -> list[GraphNode]:
+        from sampo.scheduler.utils.critical_path import critical_path_schedule_lag_optimized
+        return critical_path_schedule_lag_optimized(self.raw_wg.nodes,
+                                                    self.raw_schedule.to_schedule_work_dict,
+                                                    self.wg.nodes)
