@@ -3,11 +3,11 @@ from typing import Optional
 from operator import itemgetter
 from math import ceil
 
+from sampo.api.const import STAGE_SEP
 from sampo.schemas.graph import GraphNode, GraphEdge, WorkGraph, EdgeType
 from sampo.schemas.works import WorkUnit
 from sampo.schemas.requirements import WorkerReq, MaterialReq, ConstructionObjectReq, EquipmentReq
-
-STAGE_SEP = '_stage_'
+from sampo.utilities.priority import check_and_correct_priorities
 
 
 def make_new_node_id(work_unit_id: str, ind: int) -> str:
@@ -184,9 +184,12 @@ def split_node_into_stages(origin_node: GraphNode, restructuring_edges: list[tup
         # update id attribute with current stage node id
         wu_attrs['id'] = stage_node_id
         # update name attribute with current index of stage
-        wu_attrs['name'] = f'{wu.name}{STAGE_SEP}{stage_i}'
+        wu_attrs['model_name']['granular_name'] = f'{wu.model_name["granular_name"]}{STAGE_SEP}{stage_i}'
         # update volume attribute with passed proportion
         wu_attrs['volume'] = wu.volume * volume_proportion
+        # remove because it's unused field from Identifiable
+        if 'name' in wu_attrs:
+            del wu_attrs['name']
         # make new work unit for new stage node with updated attributes
         new_wu = WorkUnit(**wu_attrs)
         # make new graph node for new stage with created work unit and with passed edge to previous stage node
@@ -370,4 +373,7 @@ def graph_restructuring(wg: WorkGraph, use_lag_edge_optimization: Optional[bool]
                                use_lag_edge_optimization)
         fill_parents_to_new_nodes(node, id2new_nodes, restructuring_edges2new_nodes_id, use_lag_edge_optimization)
 
-    return WorkGraph(id2new_nodes[wg.start.id], id2new_nodes[wg.finish.id])
+    result = WorkGraph(id2new_nodes[wg.start.id], id2new_nodes[wg.finish.id])
+    # TODO Make structurator natively work with priorities
+    check_and_correct_priorities(result)
+    return result
