@@ -9,8 +9,7 @@ from sampo.schemas.time import Time
 
 def get_total_resources_usage(schedule: Schedule, resources_names: Iterable[str] | None = None) -> dict[str, np.ndarray]:
     df = schedule.full_schedule_df
-    points = df[['start', 'finish']].to_numpy().copy()
-    points[:, 1] += 1
+    points = df[['start', 'finish']].to_numpy()
     points = SortedList(set(points.flatten()))
     usage = defaultdict(lambda: np.zeros_like(points))
 
@@ -19,9 +18,9 @@ def get_total_resources_usage(schedule: Schedule, resources_names: Iterable[str]
 
     for swork in schedule.works:
         start = points.bisect_left(swork.start_time)
-        finish = points.bisect_left(swork.finish_time + 1)
+        finish = points.bisect_left(swork.finish_time)
         for worker in swork.workers:
-            if worker.name in resources_names or is_none:
+            if is_none or worker.name in resources_names:
                 usage[worker.name][start: finish] += worker.count
 
     return usage
@@ -48,7 +47,7 @@ def resources_sum(schedule: Schedule, resources_names: Iterable[str] | None = No
     resources_names = set(resources_names) if not is_none else {}
 
     res_sum = sum([sum([worker.count * work.duration.value for worker in work.workers
-                        if worker.name in resources_names or is_none], start=0)
+                        if worker.model_name in resources_names or is_none], start=0)
                    for work in schedule.works])
 
     return res_sum
